@@ -12,7 +12,7 @@
 namespace SymEngine
 {
 //! Complex Double Class to hold std::complex<double> values
-class ComplexDouble : public Number
+class ComplexDouble : public ComplexBase
 {
 public:
     std::complex<double> i;
@@ -30,9 +30,9 @@ public:
     virtual bool __eq__(const Basic &o) const;
     virtual int compare(const Basic &o) const;
     //! Get the real part of the complex number
-    RCP<const Number> real_part() const;
+    virtual RCP<const Number> real_part() const;
     //! Get the imaginary part of the complex number
-    RCP<const Number> imaginary_part() const;
+    virtual RCP<const Number> imaginary_part() const;
     //! \returns `false`
     // False is returned because complex cannot be compared with zero
     inline virtual bool is_positive() const
@@ -45,6 +45,11 @@ public:
     {
         return false;
     }
+    //! \returns `true`
+    inline virtual bool is_complex() const
+    {
+        return true;
+    }
     //! \return self as a double
     inline std::complex<double> as_complex_double() const
     {
@@ -82,7 +87,8 @@ public:
      * */
     RCP<const Number> addcomp(const Integer &other) const
     {
-        return make_rcp<const ComplexDouble>(i + mp_get_d(other.i));
+        return make_rcp<const ComplexDouble>(
+            i + mp_get_d(other.as_integer_class()));
     }
 
     /*! Add ComplexDoubles
@@ -90,7 +96,8 @@ public:
      * */
     RCP<const Number> addcomp(const Rational &other) const
     {
-        return make_rcp<const ComplexDouble>(i + mp_get_d(other.i));
+        return make_rcp<const ComplexDouble>(
+            i + mp_get_d(other.as_rational_class()));
     }
 
     /*! Add ComplexDoubles
@@ -123,15 +130,15 @@ public:
     virtual RCP<const Number> add(const Number &other) const
     {
         if (is_a<Rational>(other)) {
-            return addcomp(static_cast<const Rational &>(other));
+            return addcomp(down_cast<const Rational &>(other));
         } else if (is_a<Integer>(other)) {
-            return addcomp(static_cast<const Integer &>(other));
+            return addcomp(down_cast<const Integer &>(other));
         } else if (is_a<Complex>(other)) {
-            return addcomp(static_cast<const Complex &>(other));
+            return addcomp(down_cast<const Complex &>(other));
         } else if (is_a<RealDouble>(other)) {
-            return addcomp(static_cast<const RealDouble &>(other));
+            return addcomp(down_cast<const RealDouble &>(other));
         } else if (is_a<ComplexDouble>(other)) {
-            return addcomp(static_cast<const ComplexDouble &>(other));
+            return addcomp(down_cast<const ComplexDouble &>(other));
         } else {
             return other.add(*this);
         }
@@ -142,7 +149,8 @@ public:
      * */
     RCP<const Number> subcomp(const Integer &other) const
     {
-        return make_rcp<const ComplexDouble>(i - mp_get_d(other.i));
+        return make_rcp<const ComplexDouble>(
+            i - mp_get_d(other.as_integer_class()));
     }
 
     /*! Subtract ComplexDoubles
@@ -150,7 +158,8 @@ public:
      * */
     RCP<const Number> subcomp(const Rational &other) const
     {
-        return make_rcp<const ComplexDouble>(i - mp_get_d(other.i));
+        return make_rcp<const ComplexDouble>(
+            i - mp_get_d(other.as_rational_class()));
     }
 
     /*! Subtract ComplexDoubles
@@ -183,15 +192,15 @@ public:
     virtual RCP<const Number> sub(const Number &other) const
     {
         if (is_a<Rational>(other)) {
-            return subcomp(static_cast<const Rational &>(other));
+            return subcomp(down_cast<const Rational &>(other));
         } else if (is_a<Integer>(other)) {
-            return subcomp(static_cast<const Integer &>(other));
+            return subcomp(down_cast<const Integer &>(other));
         } else if (is_a<Complex>(other)) {
-            return subcomp(static_cast<const Complex &>(other));
+            return subcomp(down_cast<const Complex &>(other));
         } else if (is_a<RealDouble>(other)) {
-            return subcomp(static_cast<const RealDouble &>(other));
+            return subcomp(down_cast<const RealDouble &>(other));
         } else if (is_a<ComplexDouble>(other)) {
-            return subcomp(static_cast<const ComplexDouble &>(other));
+            return subcomp(down_cast<const ComplexDouble &>(other));
         } else {
             return other.rsub(*this);
         }
@@ -202,7 +211,8 @@ public:
      * */
     RCP<const Number> rsubcomp(const Integer &other) const
     {
-        return make_rcp<const ComplexDouble>(mp_get_d(other.i) - i);
+        return make_rcp<const ComplexDouble>(mp_get_d(other.as_integer_class())
+                                             - i);
     }
 
     /*! Subtract ComplexDoubles
@@ -210,7 +220,8 @@ public:
      * */
     RCP<const Number> rsubcomp(const Rational &other) const
     {
-        return make_rcp<const ComplexDouble>(mp_get_d(other.i) - i);
+        return make_rcp<const ComplexDouble>(mp_get_d(other.as_rational_class())
+                                             - i);
     }
 
     /*! Subtract ComplexDoubles
@@ -235,13 +246,13 @@ public:
     virtual RCP<const Number> rsub(const Number &other) const
     {
         if (is_a<Rational>(other)) {
-            return rsubcomp(static_cast<const Rational &>(other));
+            return rsubcomp(down_cast<const Rational &>(other));
         } else if (is_a<Integer>(other)) {
-            return rsubcomp(static_cast<const Integer &>(other));
+            return rsubcomp(down_cast<const Integer &>(other));
         } else if (is_a<Complex>(other)) {
-            return rsubcomp(static_cast<const Complex &>(other));
+            return rsubcomp(down_cast<const Complex &>(other));
         } else if (is_a<RealDouble>(other)) {
-            return rsubcomp(static_cast<const RealDouble &>(other));
+            return rsubcomp(down_cast<const RealDouble &>(other));
         } else {
             throw NotImplementedError("Not Implemented");
         }
@@ -252,7 +263,8 @@ public:
      * */
     RCP<const Number> mulcomp(const Integer &other) const
     {
-        return make_rcp<const ComplexDouble>(i * mp_get_d(other.i));
+        return make_rcp<const ComplexDouble>(
+            i * mp_get_d(other.as_integer_class()));
     }
 
     /*! Multiply ComplexDoubles
@@ -260,7 +272,8 @@ public:
      * */
     RCP<const Number> mulcomp(const Rational &other) const
     {
-        return make_rcp<const ComplexDouble>(i * mp_get_d(other.i));
+        return make_rcp<const ComplexDouble>(
+            i * mp_get_d(other.as_rational_class()));
     }
 
     /*! Multiply ComplexDoubles
@@ -293,15 +306,15 @@ public:
     virtual RCP<const Number> mul(const Number &other) const
     {
         if (is_a<Rational>(other)) {
-            return mulcomp(static_cast<const Rational &>(other));
+            return mulcomp(down_cast<const Rational &>(other));
         } else if (is_a<Integer>(other)) {
-            return mulcomp(static_cast<const Integer &>(other));
+            return mulcomp(down_cast<const Integer &>(other));
         } else if (is_a<Complex>(other)) {
-            return mulcomp(static_cast<const Complex &>(other));
+            return mulcomp(down_cast<const Complex &>(other));
         } else if (is_a<RealDouble>(other)) {
-            return mulcomp(static_cast<const RealDouble &>(other));
+            return mulcomp(down_cast<const RealDouble &>(other));
         } else if (is_a<ComplexDouble>(other)) {
-            return mulcomp(static_cast<const ComplexDouble &>(other));
+            return mulcomp(down_cast<const ComplexDouble &>(other));
         } else {
             return other.mul(*this);
         }
@@ -312,7 +325,8 @@ public:
      * */
     RCP<const Number> divcomp(const Integer &other) const
     {
-        return make_rcp<const ComplexDouble>(i / mp_get_d(other.i));
+        return make_rcp<const ComplexDouble>(
+            i / mp_get_d(other.as_integer_class()));
     }
 
     /*! Divide ComplexDoubles
@@ -320,7 +334,8 @@ public:
      * */
     RCP<const Number> divcomp(const Rational &other) const
     {
-        return make_rcp<const ComplexDouble>(i / mp_get_d(other.i));
+        return make_rcp<const ComplexDouble>(
+            i / mp_get_d(other.as_rational_class()));
     }
 
     /*! Divide ComplexDoubles
@@ -353,15 +368,15 @@ public:
     virtual RCP<const Number> div(const Number &other) const
     {
         if (is_a<Rational>(other)) {
-            return divcomp(static_cast<const Rational &>(other));
+            return divcomp(down_cast<const Rational &>(other));
         } else if (is_a<Integer>(other)) {
-            return divcomp(static_cast<const Integer &>(other));
+            return divcomp(down_cast<const Integer &>(other));
         } else if (is_a<Complex>(other)) {
-            return divcomp(static_cast<const Complex &>(other));
+            return divcomp(down_cast<const Complex &>(other));
         } else if (is_a<RealDouble>(other)) {
-            return divcomp(static_cast<const RealDouble &>(other));
+            return divcomp(down_cast<const RealDouble &>(other));
         } else if (is_a<ComplexDouble>(other)) {
-            return divcomp(static_cast<const ComplexDouble &>(other));
+            return divcomp(down_cast<const ComplexDouble &>(other));
         } else {
             return other.rdiv(*this);
         }
@@ -372,7 +387,8 @@ public:
      * */
     RCP<const Number> rdivcomp(const Integer &other) const
     {
-        return make_rcp<const ComplexDouble>(mp_get_d(other.i) / i);
+        return make_rcp<const ComplexDouble>(mp_get_d(other.as_integer_class())
+                                             / i);
     }
 
     /*! Divide ComplexDoubles
@@ -380,7 +396,8 @@ public:
      * */
     RCP<const Number> rdivcomp(const Rational &other) const
     {
-        return make_rcp<const ComplexDouble>(mp_get_d(other.i) / i);
+        return make_rcp<const ComplexDouble>(mp_get_d(other.as_rational_class())
+                                             / i);
     }
 
     /*! Divide ComplexDoubles
@@ -406,13 +423,13 @@ public:
     virtual RCP<const Number> rdiv(const Number &other) const
     {
         if (is_a<Rational>(other)) {
-            return rdivcomp(static_cast<const Rational &>(other));
+            return rdivcomp(down_cast<const Rational &>(other));
         } else if (is_a<Integer>(other)) {
-            return rdivcomp(static_cast<const Integer &>(other));
+            return rdivcomp(down_cast<const Integer &>(other));
         } else if (is_a<Complex>(other)) {
-            return rdivcomp(static_cast<const Complex &>(other));
+            return rdivcomp(down_cast<const Complex &>(other));
         } else if (is_a<RealDouble>(other)) {
-            return rdivcomp(static_cast<const RealDouble &>(other));
+            return rdivcomp(down_cast<const RealDouble &>(other));
         } else {
             throw NotImplementedError("Not Implemented");
         }
@@ -423,8 +440,8 @@ public:
      * */
     RCP<const Number> powcomp(const Integer &other) const
     {
-        return make_rcp<const ComplexDouble>(
-            (std::complex<double>)std::pow(i, mp_get_d(other.i)));
+        return make_rcp<const ComplexDouble>((std::complex<double>)std::pow(
+            i, mp_get_d(other.as_integer_class())));
     }
 
     /*! Raise ComplexDouble to power `other`
@@ -432,8 +449,8 @@ public:
      * */
     RCP<const Number> powcomp(const Rational &other) const
     {
-        return make_rcp<const ComplexDouble>(
-            (std::complex<double>)std::pow(i, mp_get_d(other.i)));
+        return make_rcp<const ComplexDouble>((std::complex<double>)std::pow(
+            i, mp_get_d(other.as_rational_class())));
     }
 
     /*! Raise ComplexDouble to power `other`
@@ -467,15 +484,15 @@ public:
     virtual RCP<const Number> pow(const Number &other) const
     {
         if (is_a<Rational>(other)) {
-            return powcomp(static_cast<const Rational &>(other));
+            return powcomp(down_cast<const Rational &>(other));
         } else if (is_a<Integer>(other)) {
-            return powcomp(static_cast<const Integer &>(other));
+            return powcomp(down_cast<const Integer &>(other));
         } else if (is_a<Complex>(other)) {
-            return powcomp(static_cast<const Complex &>(other));
+            return powcomp(down_cast<const Complex &>(other));
         } else if (is_a<RealDouble>(other)) {
-            return powcomp(static_cast<const RealDouble &>(other));
+            return powcomp(down_cast<const RealDouble &>(other));
         } else if (is_a<ComplexDouble>(other)) {
-            return powcomp(static_cast<const ComplexDouble &>(other));
+            return powcomp(down_cast<const ComplexDouble &>(other));
         } else {
             return other.rpow(*this);
         }
@@ -486,8 +503,8 @@ public:
      * */
     RCP<const Number> rpowcomp(const Integer &other) const
     {
-        return make_rcp<const ComplexDouble>(
-            (std::complex<double>)std::pow(mp_get_d(other.i), i));
+        return make_rcp<const ComplexDouble>((std::complex<double>)std::pow(
+            mp_get_d(other.as_integer_class()), i));
     }
 
     /*! Raise `other` to power ComplexDouble
@@ -495,8 +512,8 @@ public:
      * */
     RCP<const Number> rpowcomp(const Rational &other) const
     {
-        return make_rcp<const ComplexDouble>(
-            (std::complex<double>)std::pow(mp_get_d(other.i), i));
+        return make_rcp<const ComplexDouble>((std::complex<double>)std::pow(
+            mp_get_d(other.as_rational_class()), i));
     }
 
     /*! Raise `other` to power ComplexDouble
@@ -523,13 +540,13 @@ public:
     virtual RCP<const Number> rpow(const Number &other) const
     {
         if (is_a<Rational>(other)) {
-            return rpowcomp(static_cast<const Rational &>(other));
+            return rpowcomp(down_cast<const Rational &>(other));
         } else if (is_a<Integer>(other)) {
-            return rpowcomp(static_cast<const Integer &>(other));
+            return rpowcomp(down_cast<const Integer &>(other));
         } else if (is_a<Complex>(other)) {
-            return rpowcomp(static_cast<const Complex &>(other));
+            return rpowcomp(down_cast<const Complex &>(other));
         } else if (is_a<RealDouble>(other)) {
-            return rpowcomp(static_cast<const RealDouble &>(other));
+            return rpowcomp(down_cast<const RealDouble &>(other));
         } else {
             throw NotImplementedError("Not Implemented");
         }
