@@ -129,6 +129,20 @@ public:
         }
     }
 
+    ODictWrapper(std::map<Key, Value> &&p)
+    {
+        for (auto &iter : p) {
+            if (iter.second != Value(0)) {
+                auto erase = iter;
+                iter++;
+                p.erase(erase);
+            } else {
+                iter++;
+            }
+        }
+        dict_ = p;
+    }
+
     ODictWrapper(const Value &p)
     {
         if (p != Value(0))
@@ -252,6 +266,15 @@ public:
         return (res * tmp);
     }
 
+    template <typename FromPoly>
+    static Wrapper from_poly(const FromPoly &p)
+    {
+        Wrapper t;
+        for (auto it = p.begin(); it != p.end(); ++it)
+            t.dict_[it->first] = it->second;
+        return t;
+    }
+
     friend Wrapper operator*(const Wrapper &a, const Wrapper &b)
     {
         return Wrapper::mul(a, b);
@@ -296,7 +319,7 @@ public:
         return dict_;
     }
 
-    unsigned int size() const
+    size_t size() const
     {
         return dict_.size();
     }
@@ -334,7 +357,7 @@ umap_basic_num _find_gens_poly(const RCP<const Basic> &x);
 template <typename Container, typename Poly>
 class UPolyBase : public Basic
 {
-protected:
+private:
     RCP<const Basic> var_;
     Container poly_;
 
@@ -351,14 +374,14 @@ public:
     virtual hash_t __hash__() const = 0;
 
     // return `degree` + 1. `0` returned for zero poly.
-    virtual unsigned int size() const = 0;
+    virtual int size() const = 0;
 
     //! \returns `true` if two objects are equal
     inline bool __eq__(const Basic &o) const
     {
         if (is_a<Poly>(o))
-            return eq(*var_, *(static_cast<const Poly &>(o).var_))
-                   and poly_ == static_cast<const Poly &>(o).poly_;
+            return eq(*var_, *(down_cast<const Poly &>(o).var_))
+                   and poly_ == down_cast<const Poly &>(o).poly_;
         return false;
     }
 
@@ -397,7 +420,7 @@ public:
 
     inline int get_degree() const
     {
-        return this->poly_.degree();
+        return this->get_poly().degree();
     }
 
     static RCP<const Poly> from_dict(const RCP<const Basic> &var,
@@ -409,8 +432,8 @@ public:
 
     RCP<const Basic> as_symbolic() const
     {
-        auto it = (static_cast<const Poly &>(*this)).begin();
-        auto end = (static_cast<const Poly &>(*this)).end();
+        auto it = (down_cast<const Poly &>(*this)).begin();
+        auto end = (down_cast<const Poly &>(*this)).end();
 
         vec_basic args;
         for (; it != end; ++it) {
@@ -418,16 +441,17 @@ public:
                 args.push_back(it->second.get_basic());
             else if (it->first == 1) {
                 if (it->second == Expression(1))
-                    args.push_back(this->var_);
+                    args.push_back(this->get_var());
                 else
-                    args.push_back(mul(it->second.get_basic(), this->var_));
+                    args.push_back(
+                        mul(it->second.get_basic(), this->get_var()));
             } else if (it->second == 1)
-                args.push_back(pow(this->var_, integer(it->first)));
+                args.push_back(pow(this->get_var(), integer(it->first)));
             else
                 args.push_back(mul(it->second.get_basic(),
-                                   pow(this->var_, integer(it->first))));
+                                   pow(this->get_var(), integer(it->first))));
         }
-        if (this->poly_.empty())
+        if (this->get_poly().empty())
             args.push_back(zero);
         return SymEngine::add(args);
     }
@@ -459,9 +483,9 @@ public:
         return res;
     }
 
-    inline unsigned int get_degree() const
+    inline int get_degree() const
     {
-        return this->poly_.degree();
+        return numeric_cast<int>(this->get_poly().degree());
     }
 
     Cf get_lc() const
@@ -489,8 +513,8 @@ public:
 
     RCP<const Basic> as_symbolic() const
     {
-        auto it = (static_cast<const Poly &>(*this)).begin();
-        auto end = (static_cast<const Poly &>(*this)).end();
+        auto it = (down_cast<const Poly &>(*this)).begin();
+        auto end = (down_cast<const Poly &>(*this)).end();
 
         vec_basic args;
         for (; it != end; ++it) {
@@ -500,17 +524,17 @@ public:
                 args.push_back(integer(m));
             } else if (it->first == 1) {
                 if (m == 1) {
-                    args.push_back(this->var_);
+                    args.push_back(this->get_var());
                 } else {
                     args.push_back(
-                        Mul::from_dict(integer(m), {{this->var_, one}}));
+                        Mul::from_dict(integer(m), {{this->get_var(), one}}));
                 }
             } else {
                 if (m == 1) {
-                    args.push_back(pow(this->var_, integer(it->first)));
+                    args.push_back(pow(this->get_var(), integer(it->first)));
                 } else {
                     args.push_back(Mul::from_dict(
-                        integer(m), {{this->var_, integer(it->first)}}));
+                        integer(m), {{this->get_var(), integer(it->first)}}));
                 }
             }
         }
@@ -530,8 +554,8 @@ public:
 
     RCP<const Basic> as_symbolic() const
     {
-        auto it = (static_cast<const Poly &>(*this)).begin();
-        auto end = (static_cast<const Poly &>(*this)).end();
+        auto it = (down_cast<const Poly &>(*this)).begin();
+        auto end = (down_cast<const Poly &>(*this)).end();
 
         vec_basic args;
         for (; it != end; ++it) {
@@ -541,18 +565,18 @@ public:
                 args.push_back(Rational::from_mpq(m));
             } else if (it->first == 1) {
                 if (m == 1) {
-                    args.push_back(this->var_);
+                    args.push_back(this->get_var());
                 } else {
                     args.push_back(Mul::from_dict(Rational::from_mpq(m),
-                                                  {{this->var_, one}}));
+                                                  {{this->get_var(), one}}));
                 }
             } else {
                 if (m == 1) {
-                    args.push_back(pow(this->var_, integer(it->first)));
+                    args.push_back(pow(this->get_var(), integer(it->first)));
                 } else {
-                    args.push_back(
-                        Mul::from_dict(Rational::from_mpq(m),
-                                       {{this->var_, integer(it->first)}}));
+                    args.push_back(Mul::from_dict(
+                        Rational::from_mpq(m),
+                        {{this->get_var(), integer(it->first)}}));
                 }
             }
         }
@@ -589,8 +613,8 @@ public:
 
     std::shared_ptr<std::pair<long, Int>> operator->()
     {
-        return std::make_shared<std::pair<long, Int>>(i_,
-                                                      ptr_->get_coeff_ref(i_));
+        return std::make_shared<std::pair<long, Int>>(
+            i_, ptr_->get_coeff_ref(numeric_cast<unsigned>(i_)));
     }
 };
 
@@ -607,7 +631,8 @@ public:
     {
         this->i_++;
         while (this->i_ < this->ptr_->size()) {
-            if (this->ptr_->get_coeff_ref(this->i_) != 0)
+            if (this->ptr_->get_coeff_ref(numeric_cast<unsigned>(this->i_))
+                != 0)
                 break;
             this->i_++;
         }
@@ -628,7 +653,8 @@ public:
     {
         this->i_--;
         while (this->i_ >= 0) {
-            if (this->ptr_->get_coeff_ref(this->i_) != 0)
+            if (this->ptr_->get_coeff_ref(numeric_cast<unsigned>(this->i_))
+                != 0)
                 break;
             this->i_--;
         }
@@ -636,6 +662,12 @@ public:
     }
 };
 
+template <typename P>
+struct is_a_UPoly {
+    static const bool value
+        = std::is_base_of<UPolyBase<typename P::container_type, P>, P>::value;
+};
+
 template <typename Poly>
 RCP<const Poly> add_upoly(const Poly &a, const Poly &b)
 {
@@ -677,13 +709,6 @@ RCP<const Poly> mul_upoly(const Poly &a, const Poly &b)
     return Poly::from_container(a.get_var(), std::move(dict));
 }
 
-template <typename Poly>
-RCP<const Poly> pow_upoly(const Poly &a, unsigned int p)
-{
-    auto dict = Poly::container_type::pow(a.get_poly(), p);
-    return Poly::from_container(a.get_var(), std::move(dict));
-}
-
 template <typename Poly>
 RCP<const Poly> quo_upoly(const Poly &a, const Poly &b)
 {
